@@ -1,88 +1,16 @@
+use super::color;
+use super::color::Color;
 use super::Command;
+use super::CommandError;
 use super::ControlledCommandHandle;
 use super::Options;
 use super::OutputMessagePayload;
 
-use rand::seq::SliceRandom;
 use std::collections::HashMap;
 use std::io::Write;
 use std::sync::mpsc;
 use std::thread;
 
-#[derive(Clone)]
-pub enum Color {
-    Default,
-    Red,
-    Green,
-    Yellow,
-    Blue,
-    Magenta,
-    Cyan,
-    White,
-    Black,
-}
-
-impl Color {
-    fn get_color_list() -> Vec<Color> {
-        vec![
-            Color::Red,
-            Color::Green,
-            Color::Yellow,
-            Color::Blue,
-            Color::Magenta,
-            Color::Cyan,
-        ]
-    }
-    pub fn random() -> Color {
-        let rand_int: u32 = rand::random();
-        let chosen_variant = rand_int % 8;
-
-        match chosen_variant {
-            0 => Color::Red,
-            1 => Color::Green,
-            2 => Color::Yellow,
-            3 => Color::Blue,
-            4 => Color::Magenta,
-            5 => Color::Cyan,
-            6 => Color::White,
-            7 => Color::Black,
-            _ => panic!("Unable to generate random color"),
-        }
-    }
-
-    pub fn random_color_list(num_colors: usize) -> Vec<Color> {
-        let mut colors = Color::get_color_list();
-        let mut rng = rand::thread_rng();
-        colors.shuffle(&mut rng);
-
-        while colors.len() < num_colors {
-            let mut new_colors = colors.clone();
-            colors.append(&mut new_colors);
-        }
-
-        colors.into_iter().take(num_colors).collect()
-    }
-
-    fn open_sequence(&self) -> String {
-        match self {
-            Color::Default => "",
-            Color::Black => "\x1b[30m",
-            Color::Red => "\x1b[31m",
-            Color::Green => "\x1b[32m",
-            Color::Yellow => "\x1b[33m",
-            Color::Blue => "\x1b[34m",
-            Color::Magenta => "\x1b[35m",
-            Color::Cyan => "\x1b[36m",
-            Color::White => "\x1b[37m",
-        }
-        .to_string()
-    }
-
-    fn close_sequence(&self) -> String {
-        "\x1b[0m".to_string()
-    }
-}
-
 pub struct StandardOutCommand {
     inner_command: Command,
     color: Color,
@@ -94,7 +22,7 @@ impl StandardOutCommand {
         command: C,
         args: Cmds,
         cur_dir: Option<D>,
-    ) -> StandardOutCommand
+    ) -> Result<StandardOutCommand, CommandError>
     where
         S: AsRef<str>,
         C: AsRef<str>,
@@ -102,27 +30,27 @@ impl StandardOutCommand {
         Cmds: IntoIterator<Item = ArgType>,
         D: AsRef<str>,
     {
-        StandardOutCommand {
-            inner_command: Command::new(name, command, args, cur_dir),
-            color: Color::Default,
-        }
+        Ok(StandardOutCommand {
+            inner_command: Command::new(name, command, args, cur_dir)?,
+            color: Color::Random,
+        })
     }
 
     pub fn new_command_string<S, C, D>(
         name: S,
         command_string: C,
         cur_dir: Option<D>,
-    ) -> StandardOutCommand
+    ) -> Result<StandardOutCommand, CommandError>
     where
         S: AsRef<str>,
         C: AsRef<str>,
         D: AsRef<str>,
     {
-        let (command, args) = parse_command_string(command_string);
-        StandardOutCommand {
-            inner_command: Command::new(name, command, args, cur_dir),
-            color: Color::Default,
-        }
+        let (command, args) = parse_command_string(command_string)?;
+        Ok(StandardOutCommand {
+            inner_command: Command::new(name, command, args, cur_dir)?,
+            color: Color::Random,
+        })
     }
 
     pub fn new_command_string_with_color<S, C, D>(
@@ -130,17 +58,17 @@ impl StandardOutCommand {
         command_string: C,
         cur_dir: Option<D>,
         color: Color,
-    ) -> StandardOutCommand
+    ) -> Result<StandardOutCommand, CommandError>
     where
         S: AsRef<str>,
         C: AsRef<str>,
         D: AsRef<str>,
     {
-        let (command, args) = parse_command_string(command_string);
-        StandardOutCommand {
-            inner_command: Command::new(name, command, args, cur_dir),
+        let (command, args) = parse_command_string(command_string)?;
+        Ok(StandardOutCommand {
+            inner_command: Command::new(name, command, args, cur_dir)?,
             color,
-        }
+        })
     }
 
     pub fn new_with_color<S, C, ArgType, Cmds, D>(
@@ -149,7 +77,7 @@ impl StandardOutCommand {
         args: Cmds,
         cur_dir: Option<D>,
         color: Color,
-    ) -> StandardOutCommand
+    ) -> Result<StandardOutCommand, CommandError>
     where
         S: AsRef<str>,
         C: AsRef<str>,
@@ -157,10 +85,10 @@ impl StandardOutCommand {
         Cmds: IntoIterator<Item = ArgType>,
         D: AsRef<str>,
     {
-        StandardOutCommand {
-            inner_command: Command::new(name, command, args, cur_dir),
+        Ok(StandardOutCommand {
+            inner_command: Command::new(name, command, args, cur_dir)?,
             color,
-        }
+        })
     }
 }
 
@@ -188,12 +116,17 @@ where
         num_cmds += 1;
     }
 
+    color::populate_random_colors(&mut name_color_hash);
+
+    let verbose = options.verbose;
+    let file_handle_flags = options.file_handle_flags;
+
     let handle = super::run_commands(inner_commands, options);
 
     let recv = handle.channel;
 
     thread::spawn(move || {
-        process_channel(recv, name_color_hash, num_cmds);
+        process_channel(recv, name_color_hash, num_cmds, verbose, file_handle_flags);
     });
     ControlledCommandHandle {
         handle: handle.handle,
@@ -205,6 +138,8 @@ fn process_channel(
     chan: mpsc::Receiver<super::OutputMessage>,
     color_map: HashMap<String, Color>,
     num_cmds: usize,
+    verbose: bool,
+    file_handle_flags: bool,
 ) {
     loop {
         let message = chan.recv();
@@ -214,36 +149,56 @@ fn process_channel(
 
         let message = message.unwrap();
         let output_color = color_map.get(&message.name).unwrap();
-        let color_open_sequence = output_color.open_sequence();
-        let color_reset_sequence = output_color.close_sequence();
+        let color_open_sequence = color::open_sequence(output_color);
+        let color_reset_sequence = color::close_sequence();
+        let std_out_flag = if file_handle_flags { "(o)" } else { "" };
+        let std_err_flag = if file_handle_flags { "(e)" } else { "" };
         let mut stdout = std::io::stdout();
         let _ = stdout.write_all(color_open_sequence.as_bytes());
         let _ = match message.message {
-            OutputMessagePayload::Start => stdout.write_all(
-                format!(
-                    "{}SYSTEM: starting process {}{}\n",
-                    color_open_sequence, message.name, color_reset_sequence
-                )
-                .as_bytes(),
-            ),
-            OutputMessagePayload::Done(Some(exit_status)) => stdout.write_all(
-                format!(
-                    "{}{}:{} process exited with status: {}\n",
-                    color_open_sequence, message.name, color_reset_sequence, exit_status
-                )
-                .as_bytes(),
-            ),
-            OutputMessagePayload::Done(None) => stdout.write_all(
-                format!(
-                    "{}{}:{} process exited without exit status\n",
-                    color_open_sequence, message.name, color_reset_sequence
-                )
-                .as_bytes(),
-            ),
+            OutputMessagePayload::Start => {
+                if verbose {
+                    stdout.write_all(
+                        format!(
+                            "{}SYSTEM: starting process {}{}\n",
+                            color_open_sequence, message.name, color_reset_sequence
+                        )
+                        .as_bytes(),
+                    )
+                } else {
+                    Ok(())
+                }
+            }
+            OutputMessagePayload::Done(Some(exit_status)) => {
+                if verbose {
+                    stdout.write_all(
+                        format!(
+                            "{}{}:{} process exited with status: {}\n",
+                            color_open_sequence, message.name, color_reset_sequence, exit_status
+                        )
+                        .as_bytes(),
+                    )
+                } else {
+                    Ok(())
+                }
+            }
+            OutputMessagePayload::Done(None) => {
+                if verbose {
+                    stdout.write_all(
+                        format!(
+                            "{}{}:{} process exited without exit status\n",
+                            color_open_sequence, message.name, color_reset_sequence
+                        )
+                        .as_bytes(),
+                    )
+                } else {
+                    Ok(())
+                }
+            }
             OutputMessagePayload::Stdout(ending, mut bytes) => {
                 let mut prefix = format!(
-                    "{}{} (o):{} ",
-                    color_open_sequence, message.name, color_reset_sequence
+                    "{}{} {}:{} ",
+                    color_open_sequence, message.name, std_out_flag, color_reset_sequence
                 )
                 .into_bytes();
                 prefix.append(&mut bytes);
@@ -256,8 +211,8 @@ fn process_channel(
             }
             OutputMessagePayload::Stderr(ending, mut bytes) => {
                 let mut prefix = format!(
-                    "{}{} (e):{} ",
-                    color_open_sequence, message.name, color_reset_sequence
+                    "{}{} {}:{} ",
+                    color_open_sequence, message.name, std_err_flag, color_reset_sequence
                 )
                 .into_bytes();
                 prefix.append(&mut bytes);
@@ -279,23 +234,23 @@ fn process_channel(
     }
 }
 
-pub fn parse_command_string<S>(command: S) -> (String, Vec<String>)
+pub fn parse_command_string<S>(command: S) -> Result<(String, Vec<String>), CommandError>
 where
     S: AsRef<str>,
 {
-    let mut words = shell_words::split(command.as_ref()).unwrap();
+    let mut words = shell_words::split(command.as_ref()).map_err(|_| CommandError::ParseError)?;
     if words.is_empty() {
-        panic!("Command string contains no command");
+        return Err(CommandError::EmptyCommand);
     }
 
     let parsed_command = words.remove(0);
-    (parsed_command, words)
+    Ok((parsed_command, words))
 }
 
 #[cfg(test)]
 mod tests {
     use super::run_commands_stdout;
-    use super::Color;
+
     use super::StandardOutCommand;
     use crate::RestartOptions;
 
@@ -303,24 +258,9 @@ mod tests {
     fn run_commands() {
         let dir: Option<String> = None;
         let commands = vec![
-            StandardOutCommand::new_command_string_with_color(
-                "test1",
-                "ls -la .",
-                dir.clone(),
-                Color::Blue,
-            ),
-            StandardOutCommand::new_command_string_with_color(
-                "test2",
-                "ls -la ..",
-                dir.clone(),
-                Color::Red,
-            ),
-            StandardOutCommand::new_command_string_with_color(
-                "test3",
-                "ls -la ../..",
-                Some(".."),
-                Color::Green,
-            ),
+            StandardOutCommand::new_command_string("test1", "ls -la .", dir.clone()).unwrap(),
+            StandardOutCommand::new_command_string("test2", "ls -la ..", dir).unwrap(),
+            StandardOutCommand::new_command_string("test3", "ls -la ../..", Some("..")).unwrap(),
         ];
 
         let mut opts = super::Options::new();
